/*
 * file Activity.java
 *
 * Licensed Materials - Property of IBM
 * Restricted Materials of IBM
 * 
 * (c) Copyright IBM Corporation 2004, 2008.  All Rights Reserved. 
 * Note to U.S. Government Users Restricted Rights:  Use, duplication or  
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp. 
 */
package javax.wvcm;

import javax.wvcm.PropertyNameList.PropertyName;
import javax.wvcm.WvcmException.ReasonCode;

/**
 * A proxy for an activity resource.
 *
 * An activity resource selects a set of versions that are on a single
 * "line of descent", where a line of descent is a sequence of versions
 * connected by successor relationships.
 * Activities appear under a variety of names in existing versioning systems.
 * When an activity is used to capture a logical change, it is commonly called
 * a "change set".  When an activity is used to capture a line of descent,
 * it is commonly called a "branch".
 * 
 * @since 1.0
 */
public interface Activity extends Resource {

    /**
     * Get the workspace provider of this resource.
     * 
     * @return the {@link WorkspaceProvider} for this Resource.
     */
    public WorkspaceProvider workspaceProvider();

    /**
     * Create a new persistent activity.
     * <p>
     * Postconditions:
     * <li>(initialize-resource): A new activity exists at the location of this Resource.
     * 
     * @param feedback Specifies optional feedback to the caller.
     * @return A proxy for this new resource, whose properties are specified by feedback.
     * @throws WvcmException ReasonCode:
     * <li>{@link ReasonCode#RESOURCE_ALREADY_EXISTS_AT_LOCATION}:
     *  If a resource already 
     *  exists at the location of this Resource, the request MUST fail.
     * <li>{@link ReasonCode#CANNOT_CREATE_AT_THIS_LOCATION}:
     *  If the location of this Activity
     *  does not identify a valid location to create an activity, the request MUST fail.
     *  A client can use {@link #doCreateGeneratedResource} if it does not know a valid
     *  location for creating an activity.
     */
    public Activity doCreateResource(Feedback feedback) throws WvcmException;

    /**
     * Create a new persistent activity, where the provider can allocate the location for the
     * new activity.
     * The provider should use the client-specified location if it is valid,
     * but can select a different location if the location is not valid
     * or already identifies an activity.
     * <p>
     * Postconditions:
     * <li>(initialize-resource): A new activity resource exists at the location of this Resource.
     * 
     * @param feedback Specifies optional feedback to the caller.
     * @return A proxy for this new resource, whose properties are specified by feedback.
     * @throws WvcmException ReasonCode:
     * <li>{@link ReasonCode#METHOD_NOT_SUPPORTED}:
     *  If the provider does not support the creation of activities, this request MUST fail.
     *  A client can determine a valid location for this method
     *  with a {@link Provider#rootLocation()} request.
     */
    public Activity doCreateGeneratedResource(Feedback feedback) throws WvcmException;

    /**
     * The computed inverse of the {@link ControllableResource#ACTIVITY} property.
     */
    public static final PropertyName<ResourceList<ControllableResource>> ACTIVITY_CHECKOUT_LIST =
        new PropertyName<ResourceList<ControllableResource>>("activity-checkout-list"); //$NON-NLS-1$

    /**
     * Get the {@link #ACTIVITY_CHECKOUT_LIST} property.
     * 
     * @return the {@link #ACTIVITY_CHECKOUT_LIST} property.
     * @throws WvcmException if this Activity was not created with 
     * {@link #ACTIVITY_CHECKOUT_LIST} as a wanted property.
     */
    public ResourceList<ControllableResource> getActivityCheckoutList()
        throws WvcmException;

    /**
     * The computed inverse of the {@link Version#ACTIVITY} property.
     * Multiple versions of a single version history can be selected by an activity's
     * {@link #ACTIVITY_VERSION_LIST} property, but all {@link #ACTIVITY_VERSION_LIST} versions
     * from a given version history must be on a single line of descent from 
     * the root version of that version history. 
     */
    public static final PropertyName<ResourceList<Version>> ACTIVITY_VERSION_LIST =
        new PropertyName<ResourceList<Version>>("activity-version-list"); //$NON-NLS-1$

    /**
     * Get the {@link #ACTIVITY_VERSION_LIST} property.
     * 
     * @return the {@link #ACTIVITY_VERSION_LIST} property.
     * @throws WvcmException if this Activity was not created with
     * {@link #ACTIVITY_VERSION_LIST} as a wanted property.
     */
    public ResourceList<Version> getActivityVersionList() throws WvcmException;

    /**
     * The computed inverse of the {@link Workspace#CURRENT_ACTIVITY} and {@link Workspace#STREAM} properties
     * (i.e., a workspace is in the {#CURRENT_WORKSPACE_LIST} if it is identified in the
     * {@link Workspace#CURRENT_ACTIVITY} or {@link Workspace#STREAM} of that workspace.
     */
    public static final PropertyName<ResourceList<Workspace>> CURRENT_WORKSPACE_LIST =
        new PropertyName<ResourceList<Workspace>>("current-workspace-list"); //$NON-NLS-1$

    /**
     * Get the {@link #CURRENT_WORKSPACE_LIST} property.
     * 
     * @return the {@link #CURRENT_WORKSPACE_LIST} property.
     * @throws WvcmException if this Activity was not created with
     * {@link #CURRENT_WORKSPACE_LIST} as a wanted property.
     */
    public ResourceList<Workspace> getCurrentWorkspaceList()
        throws WvcmException;

    /**
     * The result of {@link VersionHistory#doLatestActivityVersionReport}
     * on each VersionHistory with a version in {@link #ACTIVITY_VERSION_LIST}.
     */
    public static final PropertyName<ResourceList<Version>> LATEST_VERSION_LIST =
        new PropertyName<ResourceList<Version>>("latest-version-list"); //$NON-NLS-1$

    /**
     * Get the {@link #LATEST_VERSION_LIST} property.
     * 
     * @return the {@link #LATEST_VERSION_LIST} property.
     * @throws WvcmException if this Activity was not created with
     * {@link #LATEST_VERSION_LIST} as a wanted property.
     */
    public ResourceList<Version> getLatestVersionList() throws WvcmException;

    /**
     * The tasks that this activity is performing.
     */
    public static final PropertyName<ResourceList<Task>> TASK_LIST =
        new PropertyName<ResourceList<Task>>("task-list"); //$NON-NLS-1$

    /**
     * Get the {@link #TASK_LIST} property.
     * 
     * @return the {@link #TASK_LIST} property.
     * @throws WvcmException if this Activity was not created with
     * {@link #TASK_LIST} as a wanted property.
     */
    public ResourceList<Task> getTaskList() throws WvcmException;

    /**
     * Set the {@link #TASK_LIST} property.
     * 
     * @param taskList The list of tasks that are being performed by this activity.
     * @see #getTaskList
     */
    public void setTaskList(ResourceList<Task> taskList);
}

